@@ -17,6 +17,7 @@ vardom <- function(Y, H, PSU, w_final,
                    id = NULL,  
                    Dom = NULL,
                    period = NULL,
+                   PSU_sort=NULL, 
                    N_h = NULL,
                    fh_zero=FALSE,
                    PSU_level=TRUE,
@@ -34,7 +35,7 @@ vardom <- function(Y, H, PSU, w_final,
   ### Checking
   if (length(fh_zero) != 1 | !any(is.logical(fh_zero))) stop("'fh_zero' must be the logical value")
   if (length(PSU_level) != 1 | !any(is.logical(PSU_level))) stop("'PSU_level' must be the logical value")
-  if (length(percentratio) != 1 | !any(is.integer(percentratio) | percentratio > 0)) stop("'percentratio' must be the positive integer value")
+  if (length(percentratio) != 1 | !any(is.numeric(percentratio) | percentratio > 0)) stop("'percentratio' must be the positive numeric value")
   if (length(outp_lin) != 1 | !any(is.logical(outp_lin))) stop("'outp_lin' must be the logical value")
   if (length(outp_res) != 1 | !any(is.logical(outp_res))) stop("'outp_res' must be the logical value")
   if(length(confidence) != 1 | any(!is.numeric(confidence) | confidence < 0 | confidence > 1)) {
@@ -76,7 +77,10 @@ vardom <- function(Y, H, PSU, w_final,
           if (min(q %in% names(dataset))==1) q <- dataset[, q, with=FALSE] } 
       if (!is.null(Dom)) {
           if (min(Dom %in% names(dataset))!=1) stop("'Dom' does not exist in 'dataset'!")
-          if (min(Dom %in% names(dataset))==1) Dom <- dataset[, Dom, with=FALSE] }   
+          if (min(Dom %in% names(dataset))==1) Dom <- dataset[, Dom, with=FALSE] } 
+       if (!is.null(PSU_sort)) {
+            if (min(PSU_sort %in% names(dataset))!=1) stop("'PSU_sort' does not exist in 'dataset'!")
+            if (min(PSU_sort %in% names(dataset))==1) PSU_sort <- dataset[, PSU_sort, with=FALSE] }  
      }
 
   # Y
@@ -121,6 +125,22 @@ vardom <- function(Y, H, PSU, w_final,
   } 
   np <- sum(ncol(period))
  
+  # PSU_sort
+  if (!is.null(PSU_sort)) {
+          PSU_sort <- data.frame(PSU_sort)
+          if (nrow(PSU_sort) != n) stop("'PSU_sort' must be equal with 'Y' row count")
+          if (ncol(PSU_sort) != 1) stop("'PSU_sort' must be vector or 1 column data.frame, matrix, data.table")
+          PSU_sort <- PSU_sort[, 1]
+          if (!is.numeric(PSU_sort)) stop("'PSU_sort' must be numerical")
+          if (any(is.na(PSU_sort))) stop("'PSU_sort' has unknown values")
+
+          psuag <- data.table(PSU, PSU_sort)
+          if (!is.null(period)) hpY <- data.table(period, psuag)
+          psuag <- psuag[,.N, by=names(psuag)][,N:=NULL]
+          psuag <- psuag[,.N, by=c(names(period), names(PSU))]
+          if (nrow(psuag[N>1])>0) stop("'PSU_sort' must be equal for each 'PSU'")
+  }
+
   # id
   if (is.null(id)) id <- 1:n
   id <- data.table(id)
@@ -160,7 +180,7 @@ vardom <- function(Y, H, PSU, w_final,
   }
 
   # Dom
-  namesDom <- NULL  
+  N <- namesDom <- NULL  
   if (!is.null(Dom)) {
     Dom <- data.table(Dom)
     namesDom <- names(Dom)
@@ -326,6 +346,7 @@ vardom <- function(Y, H, PSU, w_final,
                           w_final = w_final, N_h = N_h,
                           fh_zero = fh_zero,
                           PSU_level = PSU_level,
+                          PSU_sort = PSU_sort,
                           period = period,
                           dataset = NULL)
   var_est <- transpos(var_est, is.null(period), "var_est", names(period))
@@ -339,6 +360,7 @@ vardom <- function(Y, H, PSU, w_final,
                              w_final = w_design, N_h = N_h, 
                              fh_zero = fh_zero,
                              PSU_level = PSU_level,
+                             PSU_sort = PSU_sort,
                              period = period,
                              dataset = NULL)
   var_cur_HT <- transpos(var_cur_HT, is.null(period), "var_cur_HT", names(period))
@@ -483,7 +505,7 @@ vardom <- function(Y, H, PSU, w_final,
   variab <- c("respondent_count", "n_nonzero", "pop_size", "estim", "var", "se", 
               "rse", "cv", "absolute_margin_of_error", "relative_margin_of_error",
               "CI_lower", "CI_upper", "var_srs_HT",  "var_cur_HT", 
-              "var_srs_ca", "deff_sam", "deff_est", "deff")
+              "var_srs_ca", "deff_sam", "deff_est", "deff", "n_eff")
 
   setkeyv(all_result, c("nr_names", names(Dom), names(period)))
   all_result <- all_result[, c("variable", names(Dom), names(period), variab), with=FALSE]
